CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C
      program langevin
C
C mini 1D MD program
C  k_B  = 1.0
C
C
        implicit real*8 (a-h,o-z)

        real*8 mass
        logical computeRate

        pi = 3.14159265358979323846d0

C read input from file md.in
        open(11,file="md.in")
         read(11,*) nstep,nout
         read(11,*) dt
         read(11,*) x
         read(11,*) temp
         read(11,*) damp
         read(11,*) computeRate
        close(11)

        mass = 1.0d0

        write(*,*)
        write(*,*)

        call pot(0.0d0,v1,v2,e1,e2,f1,f2,g0,x01)
        e_b = e1
        call pot(x01,v1,v2,e1,e2,f1,f2,g0,x01)
        e_min = e1
c 
        dx = 0.01
        p_denom = 0.0d0
        open(51,file="POTENTIAL")

        X1 = -0.5
        X2 =  0.5
        NX = (X2-X1)/DX


        do i=0,NX
          x0 = x1 + i*dx
          call pot(x0,v1,v2,e1,e2,f1,f2,g0,x01)
          write(51,*) x0,e1
          if(x0.le.0.0) then
           p_denom = p _denom + exp(-e1/temp)*dx
          endif
        enddo
        close(51)
        write(*,20) "barrier height = ",e_b - e_min

C compute omega_bature at barrier top
        call pot(0.0d0,v1,v2,e1,e2,f1,f2,g0,x01)
        call pot(dx,v1,v2,e2,e2,f1,f2,g0,x01)
        call pot(-dx,v1,v2,e0,e2,f1,f2,g0,x01)

        curv = (-e2 + 2*e1 - e0)/(dx*dx)
        omega_b = sqrt(curv/mass)

        write(*,20) "omega_B = ",omega_b

        xa = x
        vx = (x-xa)/dt

        TAV = 0.0d0

        n_a = 0
        n_b = 0

C number of forward and backward hops
        n_ab = 0
        n_ba = 0

        write(*,*)

        write(*,30) "running Langevin dynamics with nsteps = ",nstep

C open file unit 
        open(50,file="TRAJECTORY")

        do n = 1, nstep

C compute potential and gradients
          call pot(x,v1,v2,e1,e2,f1,f2,g0,x01)

C compute random force
          call g_random(r)
          fr = sqrt(2*temp*damp*mass/dt)*r

C total force = - grad V - random force - friction 
          f = g0 + fr + damp*vx

C do integration
          xn = 2*x - xa - f*dt**2/mass
          vx = (xn-xa)/(2*dt)

          if(xn.lt.0.0) then
            n_a = n_a + 1
          else
            n_b = n_b + 1
          endif

C detect A -> B
          if(xn.gt.0.0 .and. x.lt.0.0) then
           n_ab = n_ab  +  1
          endif

C detect A -> B
          if(xn.lt.0.0 .and. x.gt.0.0) then
           n_ba = n_ba  +  1
          endif

          xa = x
          x = xn
C compute kinetic energy
          Ekin = 0.5*mass*vx*vx

C total energy = e_kin + e_pot
          E = Ekin  + e1

C kinetic temperature
          T0 = 2*Ekin

C compute average temperature
          TAV= TAV + T0

          if(mod(n,nout).EQ.0) then
           write(50,'(7f14.5)') n*dt,x,Ekin,e1,e,t0,tav/n
          endif

        enddo
C
        close(50)

        write(*,*)


        if(computeRate) then

         write(*,*)
         write(*,*)

         x_a = 0.5
         e_barr = (e_b-e_min)
         prefactor = 1.0d0/sqrt(2*pi*mass/temp)/(2*x_a)
         write(*,20) "rate(TST) = ",prefactor*exp(-e_b/temp)/p_denom

         f_kramers = (SQRT((damp/2.)**2 + omega_b**2) - damp/2.)/omega_b
         write(*,20) "Kramers prefactor = ",f_kramers
         f_kramers = f_kramers * prefactor*exp(-e_b/temp)/p_denom
         write(*,20) "rate(Kramers) = ",f_kramers

         write(*,*)

        endif

20      format(A20,f16.6)
30      format(A40,I8)

      end

C
CC definition of the potential
C
      subroutine pot(x,v1,v2,e1,e2,f1,f2,g0,x01)
      implicit real*8 (a-h,o-z)
      real*8 h(2,2),c(2,2),d(2)

        D0 = 50.5

        a1 = 1.0
        a2 = 1.0

        x01 = -0.25
        x02 =  0.25

        coup = 1.0

        v1 = D0*(1-exp(-a1*(x-x01)))**2
        f1 = 2*D0*(1-exp(-a1*(x-x01)))*exp(-a1*(x-x01))*a1

        v2 = D0*(1-exp( a2*(x-x02)))**2
        f2 = -2*D0*(1-exp(a2*(x-x02)))*exp(a2*(x-x02))*a2

        diff = (V1-V2)/SQRT((V1-V2)**2+4*coup**2)

        c(1,1) = 1.0d0/sqrt(2.)*(sqrt(1.-diff))
        c(2,1) = 1.0d0/sqrt(2.)*(sqrt(1.+diff))

        e1 = 0.5*((V1+V2)-SQRT((V1-V2)**2 + 4*coup**2))

        g0 = c(1,1)**2 * f1 + c(2,1)**2 * f2

      return
      end


      subroutine g_random(random)
      implicit none
      real*8 random,pi,x1,x2,y1,y2

        pi = 3.14159265358979323846d0

        call random_number( x1 )
        call random_number( x2 )

        y1 = sqrt( - 2.d0*log(x1) )* cos( 2 * pi * x2 )
        y2 = sqrt( - 2.d0*log(x1) )* sin( 2 * pi * x2 )

        random = y1

      return 
      end


